package dev.shadowsoffire.apotheosis.adventure.loot;

import com.mojang.datafixers.kinds.Applicative;
import dev.shadowsoffire.placebo.json.ItemAdapter;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import java.util.function.Function;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import java.util.Objects;
import java.util.Optional;
import javax.annotation.Nullable;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import net.minecraft.resources.ResourceLocation;
import java.util.Set;
import net.minecraft.world.item.ItemStack;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.adventure.compat.GameStagesCompat;
import dev.shadowsoffire.placebo.reload.WeightedDynamicRegistry;
import dev.shadowsoffire.placebo.codec.CodecProvider;

public final class AffixLootEntry implements CodecProvider<AffixLootEntry>, WeightedDynamicRegistry.ILuckyWeighted, WeightedDynamicRegistry.IDimensional, RarityClamp, GameStagesCompat.IStaged
{
    public static final Codec<AffixLootEntry> CODEC;
    protected final int weight;
    protected final float quality;
    protected final ItemStack stack;
    protected final Set<ResourceLocation> dimensions;
    protected final DynamicHolder<LootRarity> minRarity;
    protected final DynamicHolder<LootRarity> maxRarity;
    @Nullable
    protected final Set<String> stages;
    
    public AffixLootEntry(final int weight, final float quality, final ItemStack stack, final Set<ResourceLocation> dimensions, final DynamicHolder<LootRarity> min, final DynamicHolder<LootRarity> max, final Optional<Set<String>> stages) {
        this.weight = weight;
        this.quality = quality;
        this.stack = Objects.requireNonNull(stack);
        this.dimensions = dimensions;
        this.minRarity = min;
        this.maxRarity = max;
        this.stages = stages.orElse(null);
    }
    
    public AffixLootEntry(final int weight, final float quality, final ItemStack stack, final Set<ResourceLocation> dimensions, final DynamicHolder<LootRarity> min, final DynamicHolder<LootRarity> max) {
        this(weight, quality, stack, dimensions, min, max, Optional.empty());
    }
    
    public int getWeight() {
        return this.weight;
    }
    
    public float getQuality() {
        return this.quality;
    }
    
    public ItemStack getStack() {
        return this.stack.m_41777_();
    }
    
    public Set<ResourceLocation> getDimensions() {
        return this.dimensions;
    }
    
    public LootRarity getMinRarity() {
        return (LootRarity)this.minRarity.get();
    }
    
    public LootRarity getMaxRarity() {
        return (LootRarity)this.maxRarity.get();
    }
    
    public LootCategory getType() {
        return LootCategory.forItem(this.stack);
    }
    
    public Set<String> getStages() {
        return this.stages;
    }
    
    public Codec<? extends AffixLootEntry> getCodec() {
        return AffixLootEntry.CODEC;
    }
    
    static {
        CODEC = RecordCodecBuilder.create(inst -> inst.group((App)Codec.intRange(0, Integer.MAX_VALUE).fieldOf("weight").forGetter((Function)WeightedDynamicRegistry.ILuckyWeighted::getWeight), (App)PlaceboCodecs.nullableField(Codec.floatRange(0.0f, Float.MAX_VALUE), "quality", (Object)0.0f).forGetter((Function)WeightedDynamicRegistry.ILuckyWeighted::getQuality), (App)ItemAdapter.CODEC.fieldOf("stack").forGetter(a -> a.stack), (App)PlaceboCodecs.setOf(ResourceLocation.f_135803_).fieldOf("dimensions").forGetter(a -> a.dimensions), (App)RarityRegistry.INSTANCE.holderCodec().fieldOf("min_rarity").forGetter(a -> a.minRarity), (App)RarityRegistry.INSTANCE.holderCodec().fieldOf("max_rarity").forGetter(a -> a.maxRarity), (App)PlaceboCodecs.nullableField(PlaceboCodecs.setOf((Codec)Codec.STRING), "stages").forGetter(a -> Optional.ofNullable(a.stages))).apply((Applicative)inst, AffixLootEntry::new));
    }
}
